import cv2          # pip install opencv-python
import numpy as np
import matplotlib.pyplot as plt

f_img_nm = input('Input file name (*.png) = ')
img1 = cv2.imread(f_img_nm)               #   画像ファイルの読み込み
img1 = cv2.resize(img1, dsize=(400,400))  #   サイズを400×400に調整
img1 = cv2.cvtColor(img1, cv2.COLOR_BGR2RGB) / 255    #    値の範囲を[0, 1]に変換
plt.imshow(img1)
plt.show()
"""
　　　モノクロ画像を（R,G,B）の３枚用意
"""
img0 = np.empty((3, 400, 400))
for i in range(400):
    for j in range(400):
        for k in range(3):
            img0[k][i][j] = img1[i][j][k] 

"""
　　　モノクロ画像imgを、sで指定されたRGB画像として返す
"""
def select_rgb(img, s):
    t_img = np.zeros((*np.shape(img[0]),3), dtype=float)
    for i in range(len(img[0])):
        for j in range(len(img[0][0])):
            t_img[i][j][s] = img[s][i][j]
    return t_img
"""
     R、G、Bの３枚の画像、img[0],img0[1],img0[2]、の表示
"""
plt.figure(figsize=(9,3))
plt.subplot(131)
plt.title('R')
plt.imshow(select_rgb(img0, 0))
plt.subplot(132)
plt.title('G')
plt.imshow(select_rgb(img0, 1))
plt.subplot(133)
plt.title('B')
plt.imshow(select_rgb(img0, 2))

plt.show()
"""
        img0に含まれる３枚の画像の特異値分解
"""
u, s, vh = np.linalg.svd(img0, full_matrices=False)

rgb_labels = ['R', 'G', 'B']
plot_colors = ['r', 'g', 'b']
for t in range(3):
    plt.plot(range(len(s[t])), s[t], c = plot_colors[t], label = rgb_labels[t])
plt.legend(fontsize=20)
plt.show()

for t in range(3):
    plt.plot(range(20), s[t,:20], c = plot_colors[t], label = rgb_labels[t])
plt.xticks([0, 19], ['1', '20'])
plt.legend(fontsize=20)
plt.show()
"""
　　　　１番目から６番目までの特異値ごとの画像、
　　　　　　およびそれらの合成画像の作成
"""
imgs = []
for h in range(7):
    print('h =', h)
    img_1 = np.empty((400, 400, 3), dtype=float)
    for t in range(3):
        if h < 6:
            f1 = u[t, :,h:h+1] @ np.diag(s[t, h:h+1]) @ vh[t, h:h+1]
        else:
            f1 = u[t, :,:6] @ np.diag(s[t, :6]) @ vh[t, :6]
        """
　　　　　　　　　画素の値を０から１までの範囲に調整
        """
        min_f1 = np.min(f1)
        if min_f1 < 0.0:
            f1 -= min_f1
        max_f1 = np.max(f1)
        if max_f1 > 1.0:
            f1 /= max_f1
        
        for i in range(len(f1)):
            for j in range(len(f1[0])):
                img_1[i][j][t] = f1[i][j]
        
    imgs.append(img_1)

def v_s(h):                      #    h番目の特異値の和
    return np.sum(s, axis=0)[h]

"""
             特異値分解の表示
"""
fig = plt.figure(figsize=(12,8))
plt.subplot(241)              
plt.title('Original Figure')
plt.imshow(img1)            #    元画像の表示
"""
            各特異値分解および
          それらの合成画像の表示
"""
for h in range(7):
    if h < 3:
        plt.subplot(240+h+2) 
        plt.title(f's{h+1} = {v_s(h):.3f}')
        plt.imshow(imgs[h])
    else:
        plt.subplot(240+h+2) 
        plt.imshow(imgs[h])
        if h < 6:
            plt.title(f's{h+1} = {v_s(h):.3f}')
        else:
            plt.title('s1+...+s6')
fig.suptitle('Singular Value Decomposition of a Figure', fontsize=20)
fig.tight_layout()
fig.show()
